/*
    clip.c
    Copyright (C) 1997-1999 by Richard Dawe

    Clipboard functions - part of libwin

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Include files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dpmi.h>
#include <sys/movedata.h>
#include <sys/segments.h>

#include <libwin/clip.h>
#include <libwin/version.h>

/* Function declarations - for others see libwin/clip.h */
int clip_valid (void);

/* --------------
   - clip_valid -
   -------------- */

/* This checks that the clipboard can be used. When first called, it calls
   WinOldAp to find out whether it supports clipboard usage. After that it
   simply returns a stored status variable. */

int clip_valid (void)
{
    static int clip_inited = 0;
    static int clip_supported = 0;

    /* Just return the initialisation value */
    if (clip_inited) return(clip_supported);

    /* Find out whether we can use the clipboard */
    win_winoldap_getversion(NULL, NULL, &clip_supported);

    /* Done */
    clip_inited = 1;
    return(clip_supported);
}

/* -----------------
   - win_clip_open -
   ----------------- */

int win_clip_open (void)
{
    __dpmi_regs r;

    if (!clip_valid()) return(-1);

    /* Perform the call */
    memset(&r, 0, sizeof(r));

    r.x.ax = 0x1701;
    __dpmi_int(0x2F, &r);

    /* Clipboard opened ok */
    if (r.x.ax != 0)
        return(CLIP_OPENED);

    /* Clipboard already open */
    else if (r.x.ax == 0)
        return(CLIP_OPENED_ALREADY);

    /* Fail by default */
    return(-1);
}

/* ------------------
   - win_clip_empty -
   ------------------ */

int win_clip_empty (void)
{
    __dpmi_regs r;

    if (!clip_valid()) return(-1);

    /* Perform the call */
    memset(&r, 0, sizeof(r));

    r.x.ax = 0x1702;
    __dpmi_int(0x2F, &r);

    /* Clipboard emptied ok */
    if (r.x.ax != 0)
        return(0);

    /* Failed to empty it */
    return(-1);
}

/* --------------------
   - win_clip_getsize -
   -------------------- */

long win_clip_getsize (int format)
{
    __dpmi_regs r;

    if (!clip_valid()) return(-1);

    /* Perform the call */
    memset(&r, 0, sizeof(r));

    r.x.ax = 0x1704;
    r.x.dx = format;
    __dpmi_int(0x2F, &r);

    /* Failed to get size */
    if ( (r.x.ax == 0) && (r.x.dx == 0) ) return(-1);

    /* Return the size */    
    return( (r.x.dx << 16) + r.x.ax );
}

/* ------------------
   - win_clip_close -
   ------------------ */

int win_clip_close (void)
{
    __dpmi_regs r;

    if (!clip_valid()) return(-1);

    /* Perform the call */
    memset(&r, 0, sizeof(r));

    r.x.ax = 0x1708;
    __dpmi_int(0x2F, &r);

    /* Clipboard closed ok */
    if (r.x.ax != 0) return(0);

    /* Failed to close it */    
    return(-1);
}

/* --------------------
   - win_clip_getdata -
   -------------------- */

int win_clip_getdata (int format, char *buffer, long buffer_size)
{
    long clipsize;
    int paras, selector, segment;
    __dpmi_regs r;

    /* Get clipboard size - fail if this fails, or the supplied buffer is
       too small */
    clipsize = win_clip_getsize(format);
    if ( (clipsize == -1) || (clipsize < buffer_size) ) return(-1);

    /* Allocate a buffer for the library */
    paras = (clipsize + 15) >> 4;
    segment = __dpmi_allocate_dos_memory(paras, &selector);
    if (segment == -1) return(-1);

    /* Do the call */
    memset(&r, 0, sizeof(r));

    r.x.ax = 0x1705;
    r.x.dx = format;
    r.x.es = segment;
    r.x.bx = 0;         /* Already done by memset anyway, but do it again */
    __dpmi_int(0x2F, &r);

    /* Success? */
    if (r.x.ax == 0) { __dpmi_free_dos_memory(selector); return(-1); }

    /* Copy the data & free the DOS memory */
    movedata(selector, 0, _my_ds(), (int) buffer, clipsize);
    __dpmi_free_dos_memory(selector);

    /* OK */
    return(0);
}

/* --------------------
   - win_clip_setdata -
   -------------------- */

int win_clip_setdata (int format, char *buffer, long buffer_size)
{
    __dpmi_regs r;
    int segment, selector, paras;

    /* Allocate some DOS memory */
    paras = (buffer_size + 15) >> 4;
    segment = __dpmi_allocate_dos_memory(paras, &selector);
    if (segment == -1) return(-1);

    /* Copy the data into it */
    movedata(_my_ds(), (int) buffer, selector, 0, buffer_size);

    /* Copy to the clipboard */
    memset(&r, 0, sizeof(r));

    r.x.ax = 0x1703;
    r.x.dx = format;
    r.x.es = segment;
    r.x.bx = 0;                         /* Done already by memset */
    r.x.si = buffer_size >> 16;
    r.x.cx = buffer_size &  0xFFFF;
    __dpmi_int(0x2F, &r);

    /* Free the DOS memory */
    __dpmi_free_dos_memory(selector);

    /* Success? */
    if (r.x.ax == 0) return(-1); else return(0);
}

/* --------------------
   - win_clip_compact -
   -------------------- */

/* I'm not entirely sure what's meant by "compacting" the clipboard, but I
   think it discovers whether the clipboard is the size wanted. I.e. you call
   with desired size in bytes, and it returns the actual size in bytes. */

int win_clip_compact (int size)
{
    __dpmi_regs r;

    if (!clip_valid()) return(-1);

    /* Perform the call */
    memset(&r, 0, sizeof(r));

    r.x.ax = 0x1709;
    r.x.si = size >> 16;
    r.x.cx = size & 0xFFFF;
    __dpmi_int(0x2F, &r);

    /* Return the size of the largest block free */
    return((r.x.dx << 16) + r.x.ax);
}
