/*
    title.c
    Copyright (C) 1997-1999 by Richard Dawe

    DOS box titles - part of libwin.

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Inclusions */
#include <ctype.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <dpmi.h>
#include <go32.h>
#include <sys/movedata.h>

#include <libwin/title.h>

/* Defines */
#define APP_TITLE_BUFFER_LEN    80  /* Max. title length */
#define VM_TITLE_BUFFER_LEN     30  /* Max. VM length    */

/* --------------------
   - w95_setapptitle -
   -------------------- */

/* Set the application title, e.g. "MS-DOS Prompt" -> "MS-DOS Prompt - MyApp".
   This returns 1 on success, 0 on failure. */

int w95_setapptitle (char *apptitle)
{
    __dpmi_regs r;          /* Registers for function call          */
    int segment, selector;  /* Allocated memory's segment, selector */
    int paras;              /* No. paragraphs to allocate           */

    /* Check the title isn't too long */
    if ((strlen(apptitle) + 1) > APP_TITLE_BUFFER_LEN) return(0);

    /* Allocate some DOS memory */
    paras = (APP_TITLE_BUFFER_LEN + 15) >> 4;
    segment = __dpmi_allocate_dos_memory(paras, &selector);
    if (segment == -1) return(0);

    /* Put the title in it */
    movedata(_my_ds(), (int) apptitle, selector, 0, strlen(apptitle) + 1);

    /* Set up the registers - nuke DX as part of this; set AX to set the
       app title */
    memset (&r, 0, sizeof(r));
    r.x.ax = 0x168E;
    r.x.dx = 0x0000;
    r.x.es = segment;
    r.x.di = 0;

    /* Interrupt */
    __dpmi_int (0x2F, &r);

    /* Give Windows time to catch up */
    __dpmi_yield();

    /* Free the memory */
    __dpmi_free_dos_memory(selector);

    /* Return result - AX contain 1 on success, 0 on failure */
    return(r.x.ax);
}

/* -------------------
   - w95_getapptitle -
   ------------------- */

/* This gets the application title and copies maxlen-1 char's of it into
   the supplied buffer apptitle. It returns 1 on success, 0 on failure. */

int w95_getapptitle (char *apptitle, int maxlen)
{
    __dpmi_regs r;          /* Registers for function call          */
    int segment, selector;  /* Allocated memory's segment, selector */
    int paras;              /* No. paragraphs to allocate           */

    /* Allocate some DOS memory */
    paras = (APP_TITLE_BUFFER_LEN + 15) >> 4;
    segment = __dpmi_allocate_dos_memory(paras, &selector);
    if (segment == -1) return(0);

    /* Set up the registers - nuke DX as part of this; set AX to set the
       app title */
    memset (&r, 0, sizeof(r));

    r.x.ax = 0x168E;    
    r.x.dx = 0x0002;                /* Get title   */
    r.x.cx = APP_TITLE_BUFFER_LEN;  /* Buffer size */
    r.x.es = segment;               /* Segment     */
    r.x.di = 0;                     /* Offset      */

    /* Interrupt */
    __dpmi_int (0x2F, &r);

    /* Copy the string into apptitle - truncate it if there's not enough
       space */
    if (maxlen > APP_TITLE_BUFFER_LEN) maxlen = APP_TITLE_BUFFER_LEN;
    movedata(selector, 0, _my_ds(), (int) apptitle, maxlen);
    apptitle[maxlen - 1] = 0;

    /* Free the memory */
    __dpmi_free_dos_memory(selector);

    /* Return result */
    return(r.x.ax);
}

/* ------------------
   - w95_setvmtitle -
   ------------------ */

/* This sets the virtual machine title, e.g. "MS-Prompt" -> "MyVM". This
   changes the setting in the property sheet too, so it isn't that useful. It
   returns 1 on success, 0 on failure. */

int w95_setvmtitle (char *vmtitle)
{
    __dpmi_regs r;          /* Registers for function call          */
    int segment, selector;  /* Allocated memory's segment, selector */
    int paras;              /* No. paragraphs to allocate           */

    /* Check the title isn't too long */
    if ((strlen(vmtitle) + 1) > VM_TITLE_BUFFER_LEN) return(0);

    /* Allocate some DOS memory */
    paras = (VM_TITLE_BUFFER_LEN + 15) >> 4;
    segment = __dpmi_allocate_dos_memory(paras, &selector);
    if (segment == -1) return(0);

    /* Put the title in it */
    movedata(_my_ds(), (int) vmtitle, selector, 0, strlen(vmtitle) + 1);

    /* Set up the registers - nuke DX as part of this; set AX to set the
       app title */
    memset (&r, 0, sizeof(r));
    r.x.ax = 0x168E;
    r.x.dx = 0x0001;
    r.x.es = segment;
    r.x.di = 0;

    /* Interrupt */
    __dpmi_int (0x2F, &r);

    /* Give Windows time to catch up */
    __dpmi_yield();

    /* Free the memory */
    __dpmi_free_dos_memory(selector);

    /* Return result */
    return(r.x.ax);
}

/* ------------------
   - w95_getvmtitle -
   ------------------ */

/* This does the same as w95_getapptitle, but for the VM name (see above). On
   success, 1 is returned, but on failure, 0 is returned. */

int w95_getvmtitle (char *vmtitle, int maxlen)
{
    __dpmi_regs r;          /* Registers for function call          */
    int segment, selector;  /* Allocated memory's segment, selector */
    int paras;              /* No. paragraphs to allocate           */

    /* Allocate some DOS memory */
    paras = (VM_TITLE_BUFFER_LEN + 15) >> 4;
    segment = __dpmi_allocate_dos_memory(paras, &selector);
    if (segment == -1) return(0);

    /* Set up the registers - nuke DX as part of this; set AX to set the
       app title */
    memset (&r, 0, sizeof(r));

    r.x.ax = 0x168E;    
    r.x.dx = 0x0003;                /* Get title   */
    r.x.cx = VM_TITLE_BUFFER_LEN;   /* Buffer size */
    r.x.es = segment;               /* Segment     */
    r.x.di = 0;                     /* Offset      */

    /* Interrupt */
    __dpmi_int (0x2F, &r);

    /* Copy the string into apptitle - truncate it if there's not enough
       space */
    if (maxlen > VM_TITLE_BUFFER_LEN) maxlen = VM_TITLE_BUFFER_LEN;
    movedata(selector, 0, _my_ds(), (int) vmtitle, maxlen);
    vmtitle[maxlen - 1] = 0;

    /* Free the memory */
    __dpmi_free_dos_memory(selector);

    /* Return result */
    return(r.x.ax);
}

