/*
    version.c
    Copyright (C) 1997-2002 by Richard Dawe

    Windows version functions - part of libwin.

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <dos.h>
#include <dpmi.h>
#include <stdlib.h>
#include <string.h>

#include <libwin/version.h>

/* ---------------
   - win_present -
   --------------- */

/* This returns 1 if Windows is running, or 0 if it is not. */

int win_present (void) { return(win_getversion(NULL, NULL, NULL)); }

/* ------------------
   - win_getversion -
   ------------------ */

/* This fills the supplied variables with the major and minor versions and the
   mode that Windows is running in. If Windows is running it will return 1,
   else 0 is returned. */

int win_getversion (int *major, int *minor, int *mode)
{
    __dpmi_regs r;

    /* Try INT 0x2F AX=0x160A first (Win3.1 call) */
    memset(&r, 0, sizeof(r));
    r.x.ax = 0x160A;
    __dpmi_int(0x2F, &r);

    if (r.x.ax == 0) {
        /* Supported */
        if (major != NULL)  *major = r.h.bh;
        if (minor != NULL)  *minor = r.h.bl;
        if (mode != NULL)   *mode = r.x.cx;
        return(1);
    }

    /* Try INT 0x2F AX=0x1600 (Windows enhanced mode check) */
    memset(&r, 0, sizeof(r));
    r.x.ax = 0x1600;
    __dpmi_int(0x2F, &r);

    /* No Windows version running - return 0 = false */
    if ( (r.h.al == 0) || (r.h.al == 0x80) ) return(0);

    /* Windows 2.x */
    if ( (r.h.al == 0x01) || (r.h.al == 0xFF) ) {
        if (major != NULL)  *major = 2;
        if (minor != NULL)  *minor = r.h.ah;
        if (mode != NULL)   *mode = WINMODE_STANDARD; /* Assume standard */
        return(1);
    }

    /* Other version */
    if (major != NULL)  *major = r.h.al;
    if (minor != NULL)  *minor = r.h.ah;
    if (mode != NULL)   *mode = WINMODE_ENHANCED;   /* Assume it's enhanced */
    return(1);
}

/* -------------
   - win_getos -
   ------------- */

/* This returns an ID for the Windows operating system that the program is
   running on. It does more checks than win_getversion(), and can detect
   Windows NT too. */

int win_getos (void)
{
    char *os;
    int minor, major;
    union REGS r;

    /* Windows NT? Check an environment variable. Can't tell which version, so
       default to NT 3.1 */
    os = strdup(getenv("OS"));
    os = strlwr(os);
    if (strcmp(os, "windows_nt") == 0) { free(os); return(OS_WINNT31); }
    free(os);

    /* Try an INT 0x21 call to see if it's WinNT */

    /* Apparently this code is used in the MIDAS Digital Audio System timer
       code - I got this info from a newsgroup post. */
    memset(&r, 0, sizeof(r));
    r.x.ax = 0x3306;
    intdos(&r, &r);
    if ((r.x.bx & 0xFFFF) == 0x3205) return(OS_WINNT31);

    /* Use win_getversion to figure it out */
    if (win_getversion(&major, &minor, NULL)) {
        if ( (major == 4) && (minor == 0) )     return(OS_WIN95);
        if ( (major == 3) && (minor == 0) )     return(OS_WIN30);
        if ( (major == 3) && (minor == 1) )     return(OS_WIN31);
        if ( (major == 3) && (minor == 11) )    return(OS_WIN311);
        if (major == 2)                         return(OS_WIN2);
        if (major == 1)                         return(OS_WIN1);
    }

    /* Dunno */
    return(OS_UNKNOWN);
}

/* ---------------------------
   - win_winoldap_getversion -
   --------------------------- */

int win_winoldap_getversion (int *major, int *minor, int *clip)
{
    __dpmi_regs r;

    /* Call INT 0x2F AX=0x1700 to get WinOldAp version */
    memset(&r, 0, sizeof(r));
    r.x.ax = 0x1700;
    __dpmi_int(0x2F, &r);

    /* Clipboard support? */
    if (clip != NULL) { if (r.x.ax == 0x1700) *clip = 0; else *clip = 1; }

    /* Store version numbers */
    if (major != NULL)  *major = r.h.al;
    if (minor != NULL)  *minor = r.h.ah;

    /* TODO: Error check */
    /* Succeed every time because I am unsure how to detect an error */
    return(1);
}
